#!/usr/bin/perl

use warnings;
use strict;
use FindBin;
use lib "$FindBin::RealBin/../lib";
use POE::Component::IKC::ClientLite;
use TCG::Watchers::Config;
use TCG::Watchers::CTime;

my $config = TCG::Watchers::Config->load();
my $file = $config->global->status_html() or die "no status_html parameter in global config section. cannot continue.";

die "Stats publishing not activated" unless $config->global->publish;

my $server_ip = $config->global->publish_ip;
my $server_port = $config->global->publish_port;
my $name = "stats$$";

my $conn = create_ikc_client(
        ip => $server_ip,
        port => $server_port,
        name => $name,
        subscribe => [ qw(poe://watcherd/watcherd/stats) ],
);

if($conn) {
    my $data = $conn->post_respond('poe://watcherd/watcherd/stats');
    if($data) {
        open(OUT,"+>$file");
        print OUT <<HTML;
<html><head><title>ServerOp Watchers - Server Status</title></head>
<body bgcolor="white" text="black">
<h1>Server Status</h1>
<ul>    
HTML
        foreach my $server (sort keys %{$data}) {
            print OUT <<HTML2;
            <li>$server
                <ul>
                <li>ip: $data->{$server}->{ip}</li>
                <li>port: $data->{$server}->{port}</li>
                <li>state: $data->{$server}->{state}</li>
HTML2
            if($data->{$server}->{time}) {
                my $date = strftime('%R %x',localtime($data->{$server}->{time}));
                print OUT "<li>time problem noted: $date </li>\n";
            }
            if(defined($data->{$server}->{notify})) {
                print OUT "<li>Notification status: ";
                if($data->{$server}->{notify}) {
                    print OUT "sent";
                } else {
                    print OUT "not sent";
                }
                print OUT "</li>\n";
            }
            print OUT "</ul></li>\n";
        }

        print OUT <<HTML3;
</ul>
</body>
</html>
HTML3

        close OUT;
    } else {
        die "The server did not return valid data."
    }
} else {
    die "Could not create a connection to the server.";
}

exit;
