#!/usr/bin/perl

use warnings;
use strict;
use FindBin;
use lib "$FindBin::RealBin/../lib";
use POE::Component::IKC::ClientLite;
use TCG::Watchers::Config;
use TCG::Watchers::CTime;

my $config = TCG::Watchers::Config->load();

die "Stats publishing not activated" unless $config->global->publish;

my $server_ip = $config->global->publish_ip;
my $server_port = $config->global->publish_port;
my $name = "stats$$";

my $conn = create_ikc_client(
        ip => $server_ip,
        port => $server_port,
        name => $name,
        subscribe => [ qw(poe://watcherd/watcherd/stats) ],
);

if($conn) {
    my $data = $conn->post_respond('poe://watcherd/watcherd/stats');
    if($data) {
        print <<CLI;
ServerOp Watchers - Server Status
===================================

CLI
        foreach my $server (sort keys %{$data}) {
            print <<CLI2;
$server
\tip: $data->{$server}->{ip}
\tport: $data->{$server}->{port}
\tstate: $data->{$server}->{state}
CLI2
            if($data->{$server}->{time}) {
                my $date = strftime('%R %x',localtime($data->{$server}->{time}));
                print "\ttime problem noted: $date\n";
            }
            if(defined($data->{$server}->{notify})) {
                print "\tNotification status: ";
                if($data->{$server}->{notify}) {
                    print "sent";
                } else {
                    print "not sent";
                }
                print "\n";
            }
            print "\n";
        }

        print "\n\n";

    } else {
        die "The server did not return valid data."
    }
} else {
    die "Could not create a connection to the server.";
}

exit;
