# $Header: /cvsroot/serverop/watchers/lib/TCG/Watchers/Actions.pm,v 1.3 2001/07/02 19:04:09 matt Exp $

=head1 NAME

TCG::Watchers::Actions

=head1 AUTHOR

Matt Cashner (matt@cre8tivegroup.com)

=head1 DATE

$Date: 2001/07/02 19:04:09 $

=head1 REVISION

$Revision: 1.3 $

=cut

package TCG::Watchers::Actions;

use strict;
use warnings;
use Carp;

#####################
# DEFAULT ACTIONS
#####################

=head1 ABOUT ACTIONS

An action is a subroutine which processes server failure and 
reactivation notifications.  Custom actions are created from
a string which contains the code to be eval'd.  Custom actions
will be compile tested. If they fail compile testing, they will 
be thrown out, a default action will be used in its place, and
an error will be thrown.

=head1 DEFAULT ACTIONS

Default Actions are stored in lexically scoped scalars. They are 
strings which will be eval'd later by the action engine. They cannot
be called from outside this module.

=head2 Default Action List

=over 4

=item email

send an email to the addresses in the B<notify> configuration option

=item none

perform no action. this is useful if you wish for a host's information
to be available to the statistics engine or a client program but have
no real need to be harassed about the system being down.

=back

=cut

my $email_action = q!
    my %args = @_;
    my $config = $args{CONFIG};
    my $server = $args{STATUS};

    unless($config->notify) {
        warn "No list of people to send email to in Email action";
        return;
    }     
    use Mail::Sendmail;
    my $status;
    if($server->state eq 'on') {
        $status = 'BACK UP';
    } else {
        $status = 'NOT RESPONDING';
    }
    
    my $message = $server->server . " (".$config->section.") is $status on port " . $server->port . " at " . localtime($server->time);

    my %mail = ( To => $config->notify,
                 From => $config->from || 'noreply@cre8tivegroup.com',
                 Subject => 'Watcher Notification',
                 Message => $message,
               );

    sendmail(%mail);
!;

my $no_action = "1;";


sub new {
    my $class = shift;
    my $self = { CODE => { email => $email_action,
                           none => $no_action,
                         },
                 SUBREFS => { email => eval "return sub { $email_action };",
                              none => eval "return sub { $no_action };",
                            },
               };
                 
    return bless $self, $class;
}


sub create {
    my $self = shift;
    my %args = @_;
    
    return unless ($args{NAME} && $args{CODE});
    if( ($args{NAME} eq 'email') || ($args{NAME} eq 'none')) {
        carp "Cannot override default action $args{NAME}";
        return;
    }
    
    my $subref = eval "return sub { $args{CODE} };";
    
    if($@) {
        carp qq(Action $args{NAME} is not valid. Rejecting. - ERROR: $@);
        return 0;
    } else {
        $self->{SUBREFS}->{$args{NAME}} = $subref;
        $self->{CODE}->{$args{NAME}} = $args{CODE};
        return 1;
    }
}

sub subref {
    my $self = shift;
    my $action = shift or return;
    if($self->{SUBREFS}->{$action}) {
        return $self->{SUBREFS}->{$action};
    } else {
        return;
    }
}


sub code {
    my $self = shift;
    my $action = shift or return;
    if($self->{CODE}->{$action}) {
        return $self->{CODE}->{$action};
    } else {
        return;
    }
}



1;

=head1 HISTORY

    $Log: Actions.pm,v $
    Revision 1.3  2001/07/02 19:04:09  matt
    namespace change

    Revision 1.2  2001/04/04 20:55:56  matt
    custom actions should work now and default actions work as well

    Revision 1.1  2001/04/04 16:44:22  matt
    action handler. nifty eval goodness within :)


=cut
