# $Header: /cvsroot/serverop/watchers/lib/TCG/Watchers/Config.pm,v 1.11 2001/07/27 15:07:56 matt Exp $

=head1 NAME

TCG::Watchers::Config

=head1 AUTHOR

Matt Cashner (matt@cre8tivegroup.com)

=head1 DATE

$Date: 2001/07/27 15:07:56 $

=head1 REVISION

$Revision: 1.11 $

=cut

package TCG::Watchers::Config;

use 5.6.0;
use strict;
use warnings;
use TCG::Watchers::Actions;

sub _parse_config {
    my $self = shift;
    die "Empty config file" unless $self->{file};
    $self->{actions} = TCG::Watchers::Actions->new();

    # PARSE ACTIONS
    # ===================
    my %actions;

    # deven++  # for the following two lines
    # ok. a little bit of explanation here is necessary. 
    # $block is a precompiled recursive regexp. its job is to hunt down
    # the beginning of the action block '{' and the corresponding end
    # '}'.  $block is B<our>'d because $block gets defined at compile
    # time. B<my>'ing $block means that through each recursion we get a
    # new $block and that simply wont do. so we globally scope $block. 
    
    # NOTE: this regexp will fail if the user puts a '}' without a
    # matching '{' somewhere in the comments of action block.
    our $block = qr/{(?:[^{}]*(??{$block}))*[^{}]*}/;
    while($self->{file} =~ s/(?:^|\n)\s*action\s+(\S+?)\s*($block)//s) {
        my $name = $1; my $code = $2;
        next unless $code;
        $self->{actions}->create(NAME => $name, CODE => $code);
    }

    $self->{file} =~ s/#.*?\n//s;

    # PARSE GLOBAL CONFIG SECTION
    # =============================
    $self->{file} =~ s/global\s*{(.*?)}//s;
    my $global = $1;
    if($global) {
        my %global_args;
        while($global =~ s/\s*(.+?):\s+(.+?);//s) {
            my $key = $1; my $value = $2;
            $value =~ s/^"//;
            $value =~ s/"$//;
            $global_args{$key} = $value;
        }
        $self->{_global} = TCG::Watchers::Config::Section->new( section => 'global', %global_args);
    }


    # PARSE SERVER CONFIG SECTIONS
    # ==============================
    my %servers;
    while($self->{file} =~ s/server\s+(.+?)\s*{(.*?)}//s) {
        my $name = $1; my $section = $2;
        my %args;
        while ($section =~ /\s*(.+?):\s+(.+?);/sg) {
            my $key = $1; my $value = $2;
            $value =~ s/^"//;
            $value =~ s/"$//;
            $args{$key} = $value;
        }
        
        $args{'action'} = $self->{actions}->subref($args{'action'});
        
        $self->{sections}->{$name} = TCG::Watchers::Config::Section->new(section => $name, %args);
    }

    delete $self->{file}; 

}

sub load {
    my $class = shift;
    my %args = @_;
    my $self;
    my $default_conf = "<< installdir >>/bin/watchers.conf";
   
    if ($args{PATH}) {
        if( -s $args{PATH} ) {
            open (CONFIG, $args{PATH});
            {local $/; $self->{file} = <CONFIG>; }
            close CONFIG;
        } else {
            die "$args{PATH} is not valid or is empty. Cannot continue.";
        }
    } elsif(-s $default_conf) {
        open (CONFIG, $default_conf);
        {local $/; $self->{file} = <CONFIG>; }
        close CONFIG;
    } else {
        die "no valid config file found. Cannot continue.";
    }
    $self = bless($self,$class);
    $self->_parse_config();
    return $self;
}


sub section {
    my $self = shift;
    my $section = shift or return;
    if($self->{sections}->{$section}) {
        return $self->{sections}->{$section};    
    } else {
        warn "No such section ($section)";
        return;
    }
}

sub sections {
    my $self = shift;
    my @sections;
    foreach my $section (keys %{$self->{sections}}) {
        push @sections, $self->{sections}->{$section};
    }
    return @sections;
}

sub global {
    return $_[0]->{_global};
}

######################################################################

package TCG::Watchers::Config::Section;
use vars qw($AUTOLOAD);

sub new {
    my $class = shift;
    my %args = @_;
    return bless \%args, $class;
}

sub getvalue {
    my $self = shift;
    my $key = shift or return;
    if ($self->{$key}) {
        return $self->{$key};
    } else {
        return;
    }
}

sub AUTOLOAD {
    my $self = shift;
    my ($method, $ret);
    ($method = $AUTOLOAD) =~ s/.*:://;
    if($self->{$method}) {
        $ret = $self->{$method};
    } 
    return $ret;
}

1;

=head1 CREDITS

    B<Deven Corzine> - for endowing me with the wonderful regexp that
    parses the action sections. deven++

=head1 HISTORY
    
    $Log: Config.pm,v $
    Revision 1.11  2001/07/27 15:07:56  matt
    use the installdir as the default loc for the config file instead of /etc

    Revision 1.10  2001/07/25 15:47:43  matt
    reorder the way paths are specified. path is now the path + filename.

    Revision 1.9  2001/07/10 19:09:58  matt
    namespace changes

    Revision 1.8  2001/07/02 19:04:09  matt
    namespace change

    Revision 1.7  2001/04/09 19:26:40  matt
    functional action parsing


=cut
